@@ -204,7 +204,7 @@ class FieldAddButtons(BaseJsonEditWidget):
     def load_inner_widget(self):
         jsonnode = self.get_node().get_parent().get_value()
 
-        if jsonnode.get_schema_node().get_type() == 'seq':
+        if jsonnode.get_schema_node().is_type('array'):
             caption = urwid.Text("Add item: ")
         else:
             caption = urwid.Text("Add field(s): ")
@@ -258,16 +258,16 @@ class JsonWidgetNode(TreeNode):
         # don't use jsonnode.get_type() directly.
         schemanode = jsonnode.get_schema_node()
 
-        if(schemanode.get_type() == 'str'):
+        if(schemanode.is_type('string')):
             if(schemanode.is_enum()):
                 return EnumEditWidget(self)
             else:
                 return GenericEditWidget(self)
-        elif(schemanode.get_type() == 'int'):
+        elif(schemanode.is_type('integer')):
             return IntEditWidget(self)
-        elif(schemanode.get_type() == 'number'):
+        elif(schemanode.is_type('number')):
             return NumberEditWidget(self)
-        elif(schemanode.get_type() == 'bool'):
+        elif(schemanode.is_type('boolean')):
             return BoolEditWidget(self)
         else:
             return GenericEditWidget(self)
@@ -312,8 +312,7 @@ class JsonWidgetParent(ParentNode):
         else:
             jsonnode = self.get_value().get_child(key)
             schemanode = jsonnode.get_schema_node()
-            nodetype = schemanode.get_type()
-            if (nodetype == 'map') or (nodetype == 'seq'):
+            if schemanode.is_type('object') or schemanode.is_type('array'):
                 return JsonWidgetParent(jsonnode, parent=self, key=key, 
                                         depth=depth, listbox=self._listbox)
             else:
@@ -385,16 +384,15 @@ class JsonWidgetParent(ParentNode):
     def get_title_max_length(self):
         """Get max length of child titles (not counting maps and seqs)"""
         maxlen = 0
-        mytype = self.get_value().get_type()
-        if mytype == 'seq':
+        myval = self.get_value()
+        if myval.is_type('array'):
             # we need to make room for the " #10" part of "item #10"
             numchild = len(self.get_value().get_children())
             addspace = len(str(numchild)) + 2
         else:
             addspace = 0
         for child in self.get_value().get_schema_node().get_children():
-            childtype = child.get_type()
-            if not childtype == 'seq' and not childtype == 'map':
+            if not child.is_type('array') and not child.is_type('object'):
                 maxlen = max(maxlen, len(child.get_title())+addspace)
         return maxlen
 
@@ -402,10 +400,11 @@ class JsonWidgetParent(ParentNode):
 class JsonPinotFile(PinotFile):
     '''Glue to between PinotFile and underlying JSON object'''
 
-    def __init__(self, jsonfile=None, schemafile=None):
+    def __init__(self, jsonfile=None, schemafile=None, schemaobj=None):
         if jsonfile is None or os.access(jsonfile, os.R_OK):
             # file exists, and we can read it (or we're just passing "None")
-            self.json = JsonNode(filename=jsonfile, schemafile=schemafile)
+            self.json = JsonNode(filename=jsonfile, schemafile=schemafile,
+                schemanode=schemaobj)
             self.schema = self.json.get_schema_node()
         elif os.access(jsonfile, os.F_OK):
             # file exists, but can't read it
@@ -439,10 +438,9 @@ class JsonPinotFile(PinotFile):
     def get_schema_display_text(self):
         filename = self.schema.get_filename()
         if filename is None:
-            filename = "(new file)"
+            return "(auto-generated schema)"
         else:
-            filename = os.path.basename(filename)
-        return "schema: " + filename
+            return "schema: " + os.path.basename(filename)
 
 
 class JsonFrame(TreeListBox):
@@ -463,15 +461,12 @@ class JsonFileEditor(PinotFileEditor):
     JSON editor specific commands
     These routines deal with the specifics of a JSON editor.
     """
-    def __init__(self, jsonfile=None, schemafile=None, fileobj=None,
-                 program_name="JsonWidget", monochrome=True):
+    def __init__(self, jsonfile=None, schemafile=None, fileobj=None, 
+                 schemaobj=None, program_name="JsonWidget", monochrome=True):
         if fileobj is None:
-            try:
-                self.file = JsonPinotFile(jsonfile=jsonfile, schemafile=schemafile)
-            except JsonNodeError as inst:
-                sys.stderr.writelines(program_name + " error:\n")
-                sys.stderr.writelines(str(inst) + "\n\n")
-                sys.exit(2)
+            self.file = JsonPinotFile(jsonfile=jsonfile, 
+                                      schemafile=schemafile,
+                                      schemaobj=schemaobj)
         else:
             self.file = fileobj
 
@@ -486,8 +481,6 @@ class JsonFileEditor(PinotFileEditor):
         if monochrome:
             urwid.curses_display.curses.has_colors = lambda: False
 
-
-
     def handle_delete_node_request(self):
         """Handle ctrl d - "delete item"."""
         editor = self
@@ -528,7 +521,6 @@ class JsonFileEditor(PinotFileEditor):
             self.cleanup_delete_request()
             self.display_notification(msg)
 
-
     def handle_delete_node(self):
         widget, node = self.listbox.get_focus()
         node.delete_node()
